/*
 *  Land.h
 *  vagues
 *
 *  Created by Nicolas on Fri Dec 05 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 * A random land generator
 */

#ifndef _Land_h_
#define _Land_h_
#ifdef __APPLE__
#include <glut.h>
#else
#include <GL/glut.h>
#endif
#include <math.h>
#include <string>
using std::string;

#ifdef __APPLE__
#include <il.h>
#include <ilu.h>
#else
#include <IL/il.h>
#include <IL/ilu.h>
#endif
#ifdef WIN32
#include <io.h>
#endif
#include <unistd.h>

#include "constantes.h"
#include "graphicElement.h"
#include "hud.h"
#include "Menu.h"
#include "UserInput.h"
#include "clouds/cloud.h"
#include "clouds/maths.h"
using namespace Clouds;

/*
 * A grid of elevation points, calculated from a randomseed using fault algorithm, or from a tga image.
 */
class Land:public graphicElement, public MenuItemHandler, public UserInputListener{
    GLfloat * geomArray;
    GLuint * indexArray;
    int sizeGridX, sizeGridZ;
    GLfloat sizeX, sizeZ;
    
    CCloud* myCloud;
    
    void initMemory();
    void flattenLand();
    void createLandHeightsWithFaultAlgorithm(int randomseed, int nbIteration, GLfloat displacement);
    void createLandFromFile(const string& filename);
    void createLandHeightsAndColorFromFile();
    void createLandHeightsFromFile();
    void createLandNormalsFromHeights();
    void createLandColorsFromNormals();
    void addSelfShadowToColor();
    void createTextureFromCloudShadow();
    void fillIndexArray();
    GLfloat * getLandTexture(int i,int j);
    GLfloat * getLandCoord(int i,int j);
    GLfloat * getLandNormal(int i,int j);
    GLfloat * getLandColor(int i,int j);
    
    //GLuint land_id_list;
    //void buildList();
    
    Menu* LandMenu;
    virtual void userInputFunction(const string&,int questionID);

public:
        /*
         * A Land is created from a randomseed using a loop of nbIter, a each step a portion of the
         * land is elevated by displacement while the other is lowered.
         * Moreover, the size of the grid can be specified as nbCells, as the size of the terrain (size)
         * Land is assumed to be square here.
         */
        Land(int sizeGridX, int sizeGridZ, GLfloat sizeX, GLfloat sizeZ,
             CCloud* c,
             int nbIter = 100, float displacement = 0.01, int randomseed = 0);
    /*
     * Creates the Land grid from a Targa (.tga) image file.
     * If there is no alpha layer, colors are used as elevation values.
     * If there is an alpha layer, it is used as elevation values 
     * and the terrain colors are given according to the image.
     */
    Land(int sizeGridX, int sizeGridZ, GLfloat _sizeX, GLfloat _sizeZ,
         CCloud* c,
         const string& heightfilename);
    ~Land();
    /*
     *interpolates to get the heigt of any point.
     * return 0 when out of bounds.
     */
    GLfloat getHeigt(GLfloat x, GLfloat z);
//    void print();
    virtual void menuItemClicked(int MenuEntry);
protected:
        virtual void drawMe();
};

#endif //_Land_h_


