/*
 *  Sea.cpp
 *  Waves
 *
 *  Created by Nicolas on Wed Nov 26 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */

#include "Sea.h"

#define SEA_ADDNULLWAVE_MENUENTRY 0
#define SEA_ADDRANDOMWAVE_MENUENTRY 1
#define SEA_KBCONTROL_MENUENTRY 2
//******CONSTRUCTOR*****
Sea::Sea(int _nbCellsX, int _nbCellsZ, GLfloat _sizeX, GLfloat _sizeZ){
    this->nbCellsX = _nbCellsX;
    this->nbCellsZ = _nbCellsZ;
    this->sizeX = _sizeX;
    this->sizeZ = _sizeZ;
    init();
}
Sea::~Sea(){
    if(this->geomArray == NULL)
        delete [] this->geomArray;
    if(this->indexArray == NULL)
        delete [] this->indexArray;
}
void Sea::init(){
    geomArray = new GLfloat[6*(nbCellsX+1)*(nbCellsZ+1)];
    indexArray = new GLuint[4*(nbCellsX)*(nbCellsZ)];
    
    SeaMenu = new Menu("Mer");
    StaticMenu::currentMenu->addSubMenu(SeaMenu);
    SeaMenu->addSubMenu(new Menu("nouvelle vague nulle",this, SEA_ADDNULLWAVE_MENUENTRY));
    SeaMenu->addSubMenu(new Menu("nouvelle vague aléatoire",this, SEA_ADDRANDOMWAVE_MENUENTRY));
    SeaMenu->addSubMenu(new Menu("controler la mer au clavier",this, SEA_KBCONTROL_MENUENTRY));
        
    this->noiseFactor = 0.0;
    this->previousframetime = 0;
}

//******Wave access****
void Sea::addWave(Wave* v){
    this->myWaves.insert(this->myWaves.end(),v);
    hud::currentHud->addMessage(new TemporizedMessage(v->toString()));
    v->setSea(this);
}
void Sea::removeWave(Wave * v){
    myWaves.erase(remove(this->myWaves.begin(),this->myWaves.end(),v));
}

//*****Each frame****
void Sea::update(){
    GLfloat *ptr = geomArray;
    GLuint ind, *ptri = indexArray;
    int time = SimuParams::getOldTime();
    for(int i=0;i<nbCellsX+1;i++){
        for(int j=0;j<nbCellsZ+1;j++){
            *ptr=*(ptr+1)=*(ptr+2)=*(ptr+3)=*(ptr+5)=0.0f;
            *(ptr+4)=1.0f;
            for(unsigned short v=0;v<this->myWaves.size();++v){
                myWaves[v]->fillData(i*sizeX/nbCellsX,j*sizeZ/nbCellsZ,
                                     time/1000.0,ptr,
                                     sizeX/nbCellsX,/**(ptr+5),*/previousframetime);
            }
            *(ptr+3)+=i*sizeX/nbCellsX;
            *(ptr+4) -= 9;
            *(ptr+5)+=j*sizeZ/nbCellsZ;
/*            if (bDrawNormals)
            {
                glPushAttrib(GL_LIGHTING_BIT);
                glDisable(GL_LIGHTING);
                glBegin(GL_LINES);
                glColor3fv(fullred);
                glVertex3f(*(ptr+3),*(ptr+4),*(ptr+5));
                glVertex3f(*(ptr+3)+*(ptr+0),*(ptr+4)+*(ptr+1),*(ptr+5)+*(ptr+2));
                glEnd();
                glPopAttrib();
            }*/
            ptr+=6;
            if(i != nbCellsX && j!=nbCellsZ){
                ind = j+i*(nbCellsZ+1);
                *ptri = ind; ptri++;
                *ptri = ind+1; ptri++;
                *ptri = ind+nbCellsZ+2; ptri++;
                *ptri = ind+nbCellsZ+1;ptri++;
            }
            
        }
    }
    this->previousframetime = time;
}

void Sea::drawMe(){
    glPushMatrix();
    glPushAttrib(GL_LIGHTING_BIT);
    glDisable(GL_COLOR_MATERIAL);
    glMaterialfv(GL_FRONT,GL_AMBIENT,transparentblue);
    glMaterialfv(GL_FRONT,GL_DIFFUSE,transparentgrey);
    glMaterialfv(GL_FRONT,GL_SPECULAR,fullwhite);
    glMaterialf(GL_FRONT,GL_SHININESS,50);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    this->update();
    glEnableClientState(GL_VERTEX_ARRAY);
    glInterleavedArrays(GL_N3F_V3F,0,geomArray);
    glDrawElements(GL_QUADS,4*(nbCellsX)*(nbCellsZ),GL_UNSIGNED_INT,indexArray);
    glDisable(GL_BLEND);
    glDisableClientState(GL_VERTEX_ARRAY);
    glPopAttrib();
    glPopMatrix();
}


/*************Keyboard Functions*****************/

void Sea::keyboardFunc(unsigned char c, int x, int y){
    stringstream smsg;
    switch(c)
    { 
        case ')':this->incNoiseFactor();
            smsg<<"NoiseFactor:"<<this->getNoiseFactor();
            hud::currentHud->addMessage(new TemporizedMessage(smsg.str()));
            break;
        case '-':this->decNoiseFactor();
            smsg<<"NoiseFactor:"<<this->getNoiseFactor();
            hud::currentHud->addMessage(new TemporizedMessage(smsg.str()));
            break;
        case 'n':
            this->removeWave(0);
            this->addWave(new Wave(SimuParams::getNewTime()));
            break;
/*        case 'N':
            if (bDrawNormals)
                if(bDrawTangents)
                    bDrawNormals = bDrawTangents = FALSE;
                else
                    bDrawTangents = TRUE;
            else bDrawNormals = TRUE;
            break;
            */
    }
}
/**Menu handler*/
void Sea::menuItemClicked(int menuEntry){
    switch(menuEntry){
        case SEA_ADDNULLWAVE_MENUENTRY:
            this->addWave(new Wave());
            break;
        case SEA_ADDRANDOMWAVE_MENUENTRY:
            this->addWave(new Wave(SimuParams::getNewTime()));
            break;
        case SEA_KBCONTROL_MENUENTRY:
            Keyboard::selectKeyboardMode(this);    
        default:break;
    }
}


