/*
 *  Wave.cpp
 *  Waves
 *
 *  Created by Nicolas on Fri Nov 28 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */

#include "Wave.h"

#define WAVE_INFO_MENUENTRY 0
#define WAVE_DELETE_MENUENTRY 1
#define WAVE_KBCONTROL_MENUENTRY 2
//*******CONSTRUCTORS*****
Wave::Wave(unsigned int randomseed){
    srand(randomseed);
    for(int i = 0;i<10;i++) rand();
    this->a = (float)rand()/(RAND_MAX)/2;
    this->dir = ((float)rand()/(RAND_MAX)*2);
    this->k = ((float)rand()/(RAND_MAX)*2);
    this->topLambda = .5+2*((float)rand()/RAND_MAX);
    this->init();
}    
Wave::Wave():
a(0),
dir(0),
k(0),
topLambda(.1){
    this->init();
}
Wave::Wave(float _a, float _dir, float _k, float _topLambda):
a(_a),
dir(_dir),
k(_k),
topLambda(_topLambda){
    this->init();
}
void Wave::init(){
    this->w = sqrt(GRAVITY*k);
    this->lagPhiFactor = 0.00;    
    modifParams = 4;
}
void Wave::setSea(Sea* s){
    this->mySea = s;
    waveMenu = new Menu("Vague");
    this->mySea->SeaMenu->addSubMenu(waveMenu);
    waveMenu->addSubMenu(new Menu("Infos",this, WAVE_INFO_MENUENTRY));
    waveMenu->addSubMenu(new Menu("Supprimer",this, WAVE_DELETE_MENUENTRY));
    waveMenu->addSubMenu(new Menu("Controler au clavier",this, WAVE_KBCONTROL_MENUENTRY));
}


Wave::~Wave(){
    if(this->waveMenu != NULL)
        this->mySea->SeaMenu->removeSubMenu(this->waveMenu);
}

//*********toString****
string Wave::toString(){
    stringstream res;
    res<<"infos: amplitude "<<a<<", direction "<<dir<<", nombre d'onde "<<k<<", freq. angulaire "<<w;
    return res.str();
}

//****** THE THING*****

void Wave::fillData(float s,float t, float time, GLfloat *ptr,float delta,float oldtime){
    GLfloat x,y,z, tsx,tsy,tsz, ttx,tty,ttz, nx,ny,nz;
    GLfloat phaseAngle = s*k*getCos(dir)+t*k*getSin(dir) - w*time;
    y = -a*getCos(phaseAngle);
    GLfloat phaseAngleWithAccel = (y>0)?phaseAngle - topLambda*y:phaseAngle;
    x = s+a*getCos(dir)*getSin(phaseAngleWithAccel);
    z = t+a*getSin(dir)*getSin(phaseAngleWithAccel);

    
    delta /= 10;
    GLfloat phaseAngleTs = (s+delta)*k*getCos(dir)+t*k*getSin(dir) - w*time;
    tsy = -a*getCos(phaseAngleTs);
    GLfloat phaseAngleTsWithAccel = (tsy>0)?phaseAngleTs - topLambda*tsy:phaseAngleTs;
    tsx = s+delta+a*getCos(dir)*getSin(phaseAngleTsWithAccel);
    tsz = t+a*getSin(dir)*getSin(phaseAngleTsWithAccel);
    
    GLfloat phaseAngleTt = s*k*getCos(dir)+(t+delta)*k*getSin(dir) - w*time;
    tty = -a*getCos(phaseAngleTt);
    GLfloat phaseAngleTtWithAccel = (tty>0)?phaseAngleTt - topLambda*tty:phaseAngleTt;
    ttx = s+a*getCos(dir)*getSin(phaseAngleTtWithAccel);
    ttz = t+delta+a*getSin(dir)*getSin(phaseAngleTtWithAccel);
    
    tsx -= x; tsy -= y; tsz -= z; 
    ttx -= x; tty -= y; ttz -= z;
    /* calcul de la tengante par la dérivée... erreur ??
    tsx = 1+a*getCos(dir)*getSin(phaseAngle);
    tsy = 0+a*getSin(dir)*getSin(phaseAngle);
    tsz = -a*getCos(phaseAngle);
    ttx = 0+a*getCos(dir)*getSin(phaseAngle);
    tty = 1+a*getSin(dir)*getSin(phaseAngle);
    ttz = -a*getCos(phaseAngle);
    */
    nx = tsy*ttz - tty*tsz;
    ny = tsz*ttx - ttz*tsx;
    nz = tsx*tty - ttx*tsy;
    
    GLfloat d = sqrt(nx*nx+ny*ny+nz*nz);
    nx/=d; ny/=d; nz/=d;
    *ptr-=nx; *(ptr+1)-=ny; *(ptr+2)-=nz;
    *(ptr+3)+=x-s; *(ptr+4)+=y; *(ptr+5)+=z-t;
    
}


//***********Keyboard Functions*****
void Wave::keyboardFunc(unsigned char c, int x, int y){
    stringstream smsg;
    switch(c)
    { 
        case ' ':
            modifParams --;
            if (modifParams == 0){
                modifParams = 6;
            }
            switch(modifParams){
                case 6:
                    smsg <<"Modification Amplitude:"<<this->a;
                    break;
                case 5:
                    smsg<<"Modification direction dir:"<<this->dir;
                    break;
                case 4:
                    smsg<<"Modification Nombre d'onde k:"<<this->k;
                    break;
                case 3:
                    smsg<<"Modification freq.angulaire w:"<<this->w;
                    break;                    
                case 2:
                    smsg<<"Modification LagPhiFactor:"<<this->lagPhiFactor;
                    break;
                case 1:
                    smsg<<"Modification TopLambda:"<<this->topLambda;
                    break;
                default:smsg<<"erreur dans Wave:modifParams incorrect\n";
            }
            break;
        case 'o':
            switch(modifParams){
                case 6:
                    this->a *=1.05;
                    smsg<<"Amplitude:"<<this->a;
                    break;
                case 5:
                    this->dir +=.1;
                    smsg<<"direction dir:"<<this->dir;
                    break;
                case 4:
                    this->k *=1.05;
                    smsg<<"Nombre d'onde k:"<<this->k;
                    break;
                case 3:
                    this->w *=1.05;
                    smsg<<"Frequence angulaire w:"<<this->w;
                    break;
                case 2:
                    this->lagPhiFactor *=1.05;
                    smsg<<"LagPhiFactor:"<<this->lagPhiFactor;
                    break;
                case 1:
                    this->topLambda *=1.1;
                    smsg<<"TopLambda:"<<this->topLambda;
                    break;
                default:smsg<<"erreur dans Wave:modifParams incorrect\n";
            }
            break;
        case 'p':
            switch(modifParams){
                case 6:
                    this->a /=1.05;
                    smsg<<"Amplitude:"<<this->a;
                    break;
                case 5:
                    this->dir -=.1;
                    smsg<<"Nombre d'onde dir:"<<this->dir;
                    break;
                case 4:
                    this->k /=1.05;
                    smsg<<"Nombre d'onde k:"<<this->k;
                    break;
                case 3:
                    this->w /=1.05;
                    smsg<<"Frequence angulaire w:"<<this->w;
                    break;
                case 2:
                    this->lagPhiFactor /=1.05;
                    smsg<<"LagPhiFactor:"<<this->lagPhiFactor;
                    break;
                case 1:
                    this->topLambda /=1.1;
                    smsg<<"TopLambda:"<<this->topLambda;
                    break;
                default:smsg<<"erreur dans Wave:modifParams incorrect\n";
            }
            break;
        case 'i':
            smsg<<this->toString();
            break;
    }
    hud::currentHud->addMessage(new TemporizedMessage(smsg.str()));
}

/**Menu handler*/
void Wave::menuItemClicked(int menuEntry){
    switch(menuEntry){
        case WAVE_INFO_MENUENTRY:
            hud::currentHud->addMessage(new TemporizedMessage(this->toString()));
            break;
        case WAVE_DELETE_MENUENTRY:
            if(this->mySea != NULL)
                this->mySea->removeWave(this);
            hud::currentHud->addMessage(new TemporizedMessage("Suppression vague"));
            delete this;
            break;
        case WAVE_KBCONTROL_MENUENTRY:
            Keyboard::selectKeyboardMode(this);
            break;
        default:break;
    }
}


