/*------------------------------------------------------------------------------*/
/** 
*  \file  cloud.h
*  \brief Definition of class \c CCloud
*  \author Antoine Bouthors 2004-01-09
*/ 
/*------------------------------------------------------------------------------*/

#ifndef __cloud_h_
#define __cloud_h_

#include "config.h"
#include "field3d.h"
#include "billboard.h"
#include "maths.h"
#include "Menu.h"
#include "../SimuParams.h"

namespace Clouds {
    
    
    
    struct CShell
{
    float Distance; 
    T_TriangleVector Triangles;
};

typedef vector<CShell> T_ShellVector;
typedef T_ShellVector::iterator IT_ShellVector;
typedef T_ShellVector::const_iterator CIT_ShellVector;
typedef T_ShellVector::const_reverse_iterator CRIT_ShellVector;



/*------------------------------------------------------------------------------*/
/** 
*  \class  CCloud
*  \brief  PUT THE COMMENTS ON THE CLASS HERE
*  \author Antoine Bouthors 2004-01-09
*  PUT FULL DESCRIPTION HERE 
*/ 
/*------------------------------------------------------------------------------*/

class CCloud:public MenuItemHandler,public KeyboardMode
{
    
    //Init
public:
    CCloud( ulong NbBillboards, float BillboardSize = 2 );
    virtual ~CCloud();
    
    inline void SetBillboardSize( float BillboardSize )
    {
        m_BillboardSize = BillboardSize;
    }
    inline void SetProjParams( float ProjFOV, float ProjAspect, float ProjZNear )
    {
        if(	m_ProjFOV != ProjFOV 
                || m_ProjAspect != ProjAspect 
                || m_ProjZNear != ProjZNear )
        {
            m_ProjFOV = ProjFOV;
            m_ProjAspect = ProjAspect;
            m_ProjZNear = ProjZNear;
            CreateShafts();
        }
    }
    void CreateBillboards( int NbBillboards );
    void Init( ulong XSize, ulong YSize, ulong ZSize );
private:
        void CreateShafts();
    CShell CreateShell( float Distance );
    void SubdivideShell( const CShell& Old, CShell& New );
    
    ulong m_NbBillboards;
    float m_BillboardSize;
    long m_XSize;
    long m_YSize;
    long m_ZSize;
    
    float m_ProjAspect;
    float m_ProjFOV;
    float m_ProjZNear;
    
    float m_ShaftMaxDistance;
    float m_ShaftStepSize;
    int m_ShaftSubdivisionLevel;
    float m_ShaftClipBottom;
    float m_ShaftClipTop;
    
    //update
public:
    void SmoothField( int SmoothSize );
    void Shade();
    void sortDistMapEye();
private:
        struct CElement { ulong x, y, z; char Billboard; float Color; };
        typedef multimap<float,CElement> T_DistMap;
    T_BillbardVector	m_Billboards;
    
    T_DistMap			m_DistMapFromSun;
    T_DistMap			m_DistMapFromEye;
    
        //render
public:
        void Draw();
    CTexture& Shadow() { return m_Shadow; }
    inline void GetShadowTexCoord( float x, float y, float z, CPoint& t )
    {
        GetShadowTexCoord( CPoint( x, y, z ), t );
    }
    void GetShadowTexCoord( const CPoint& p, CPoint& t );
private:
        inline void UnRotateModelViewMatrix()
    {
            float matrix[4][4];
            // "Drotationne" la matrice modelview
            glGetFloatv( GL_MODELVIEW_MATRIX, &matrix[0][0] );
            matrix[0][0] = matrix[1][1] = matrix[2][2] = 1;
            matrix[0][1] = matrix[0][2] = 0;
            matrix[1][0] = matrix[1][2] = 0;
            matrix[2][0] = matrix[2][1] = 0;
            glLoadMatrixf( &matrix[0][0] );
    }
    void StoreSunMatrix();
    void RenderShell( const CShell& Shell );
    inline float AtmFunction( float Height )
    {
        float t = expf( (float)Height*.01f );
        if( t>1 ) t=1;
        if( t<0 ) t=0;
        return t;
    }

    CTexture m_Shadow;
    CMatrix m_SunMatrix;
    T_ShellVector m_Shells;
      
    //model
public:
    void update();
    void createRandomClouds();
    void initModel();
    int windVelocity(int k);
private:
        
    T_FloatField m_Field;
    struct cloudpoint{
        cloudpoint(){};
        cloudpoint(bool h,bool c,bool a){hum = h;cld = c;act = a;}
        bool hum;bool cld;bool act;
    };
    
    unsigned char currentCloudModel;
    Field3D<cloudpoint> cloudModel[2];
    T_FloatField extProbField;
    T_FloatField humProbField;
    T_FloatField actProbField;
    
    int frameWindCounter;
    bool activeNear(int i,int j, int k, int modelID);
    
    //Usercontrol
public:
        virtual void menuItemClicked(int MenuEntry);
    virtual void keyboardFunc(unsigned char c, int x, int y);
private:
        void initControlState();
    Menu* cloudMenu;
    
    bool myDrawQuality;
    bool myRunningFlag;
    bool myLoopingFlag;
};


} // namespace Clouds

#endif // __cloud_h_

