#ifndef _FIELD3D_H_
#define _FIELD3D_H_

#include <vector>

using std::vector;

namespace Clouds {

template < class T >
class Field3D
{
public:
	typedef std::vector<T>::iterator iterator;
	typedef std::vector<T>::const_iterator const_iterator;

	Field3D( const vector<T>& Contents, unsigned int W, unsigned int H, unsigned int D )
	:	m_Contents( Contents ),
		m_Width( W ),
		m_Height( H ),
		m_Depth( D )
	{}

	Field3D( unsigned int W, unsigned int H, unsigned int D )
	:	m_Width( W ),
		m_Height( H ),
		m_Depth( D )
	{
		m_Contents.resize( W*H*D );
	}

	Field3D()
	:	m_Width( 0 ),
		m_Height( 0 ),
		m_Depth( 0 )
	{}

	void Init( const vector<T>& Contents, unsigned int W, unsigned int H, unsigned int D )
	{
		m_Contents = Contents;
		m_Width = W;
		m_Height = H;
		m_Depth = D;
	}

	void Init( unsigned int W, unsigned int H, unsigned int D )
	{
		m_Contents.resize( W*H*D );
		m_Width = W;
		m_Height = H;
		m_Depth = D;
	}

	inline const T& Access( unsigned int i, unsigned int j, unsigned int k ) const
	{
		return m_Contents[ i + m_Width*( j + m_Height*k ) ];
	}

	inline T& Access( unsigned int i, unsigned int j, unsigned int k )
	{
		return m_Contents[ i + m_Width*( j + m_Height*k ) ];
	}

	inline void Fill( const T& Val )
	{
		m_Contents.assign( m_Width*m_Height*m_Depth, Val );
	}

	inline const T& Front() const { return m_Contents.front(); }
	inline T& Front() { return m_Contents.front(); }


	inline iterator begin() { return m_Contents.begin(); }
	inline iterator end() { return m_Contents.end(); }

	inline const_iterator begin() const { return m_Contents.begin(); }
	inline const_iterator end() const { return m_Contents.end(); }

	inline unsigned int Width() const { return m_Width; }
	inline unsigned int Height() const { return m_Height; }
	inline unsigned int Depth() const { return m_Depth; }

private:

	vector<T>		m_Contents;
	unsigned int	m_Width;
	unsigned int	m_Height;
	unsigned int	m_Depth;
};

typedef Field3D<unsigned char> T_CharField;
typedef Field3D<int> T_BoolField;
typedef Field3D<float> T_FloatField;
typedef Field3D<double> T_DoubleField;

} // namespace Clouds

#endif //_FIELD3D_H_
