/*------------------------------------------------------------------------------*/
/** 
*  \file  maths.h
*  \brief Definition of several mathematic classes : CPoint, CMatrix, CTriangle
*  \author Antoine Bouthors 2004-01-24
*/ 
/*------------------------------------------------------------------------------*/

#ifndef __maths_h_
#define __maths_h_

#include "config.h"

namespace Clouds {
    
    
    /*------------------------------------------------------------------------------*/
    /** 
    *  \class  CPoint
    *  \brief  PUT THE COMMENTS ON THE CLASS HERE
    *  \author Antoine Bouthors 2004-01-24
    *  PUT FULL DESCRIPTION HERE 
    */ 
    /*------------------------------------------------------------------------------*/
    
    struct CPoint
{
    CPoint() {}
    CPoint( float X, float Y, float Z ){x=X;y=Y;z=Z;}
    CPoint operator+( const CPoint& p ) const { return CPoint( x+p.x, y+p.y, z+p.z ); }
    CPoint operator-( const CPoint& p ) const { return CPoint( x-p.x, y-p.y, z-p.z ); }
    CPoint operator^( const CPoint& p ) const 
{
    return CPoint( y*p.z - z*p.y,
                   z*p.x - x*p.z,
                   x*p.y - y*p.x ); 
}
CPoint operator/( float f ) const { return CPoint( x/f, y/f, z/f ); }
void operator/=( float f ) { x/=f; y/=f; z/=f; }
void operator*=( float f ) { x*=f; y*=f; z*=f; }
operator const float*() const { return &x; }
float Norm() const { return sqrt( x*x + y*y + z*z ); }
void Normalize( ) { (*this)/=Norm(); }

float x, y, z; 
};

struct CMatrix
{
    float M[4][4];
    
    void LoadIdentity() 
    { 
        memset( M, 0, sizeof(M) ); 
        M[0][0] = M[1][1] = M[2][2] = M[3][3] = 1;
    }
    void LoadFromModelview() { glGetFloatv( GL_MODELVIEW_MATRIX, &M[0][0] ); }
    void LoadFromProjection() { glGetFloatv( GL_PROJECTION_MATRIX, &M[0][0] ); }
    void Inverse()
    {
        float C[4][4];
        for( int j=0 ; j<4 ; ++j )
        {
            for( int i=0 ; i<4 ; ++i )
            {
                C[i][j] = M[j][i];
            }
        }
        
        C[3][0] = - M[3][0]*C[0][0] - M[3][1]*C[1][0] - M[3][2]*C[2][0];
        C[3][1] = - M[3][0]*C[0][1] - M[3][1]*C[1][1] - M[3][2]*C[2][1];
        C[3][2] = - M[3][0]*C[0][2] - M[3][1]*C[1][2] - M[3][2]*C[2][2];
        
        memcpy( M, C, sizeof(M) );
    }
    
    CPoint operator*( const CPoint& p )
    {
        return CPoint( M[0][0]*p.x
                       + M[1][0]*p.y
                       + M[2][0]*p.z
                       + M[3][0],
                       M[0][1]*p.x
                       + M[1][1]*p.y
                       + M[2][1]*p.z
                       + M[3][1],
                       M[0][2]*p.x
                       + M[1][2]*p.y
                       + M[2][2]*p.z
                       + M[3][2]
                       );
    }
    
    CMatrix operator*( const CMatrix& m )
    {
        CMatrix R;
        for( int j=0 ; j<4 ; ++j )
        {
            for( int i=0 ; i<4 ; ++i )
            {
                R.M[j][i] = M[j][0] * m.M[0][i]
                +	M[j][1] * m.M[1][i]
                +	M[j][2] * m.M[2][i]
                +	M[j][3] * m.M[3][i];
                
            }
        }
        return R;
    }
};



struct CTriangle 
{
    CTriangle() {}
    CTriangle( const CPoint& p1, const CPoint& p2, const CPoint& p3 )
    : a(p1), b(p2), c(p3)
    {}
    
    CPoint a, b, c;
    
};

typedef vector<CTriangle> T_TriangleVector;
typedef T_TriangleVector::iterator IT_TriangleVector;
typedef T_TriangleVector::const_iterator CIT_TriangleVector;


} // namespace Clouds

#endif // __maths_h_
