/*------------------------------------------------------------------------------*/
/** 
 *  \file  CTexture.cpp
 *  \brief Definition of class \c CTexture
 *  \author Antoine Bouthors 2002-10-31
 */ 
/*------------------------------------------------------------------------------*/

#include "texture.h"

using namespace Clouds;

/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture constructor
 *
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
CTexture::CTexture():
	m_TexData( NULL ),
	m_DataSize( 0 ),
	m_Width( 0 ),
	m_Height( 0 ),
	m_TileModeS( TileMode_Tile ),
	m_TileModeT( TileMode_Tile ),
	m_EnvMode( EnvMode_Modulate ),
	m_UseMipMap( true ),
	m_Filtering( Filtering_Bilinear ),
	m_CubeMapIndex( -1 ),
	m_OpenGLId( 0 ),
	m_HasChanged( false )
{	
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture constructor
 *
 *  \param  File EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
CTexture::CTexture( const uchar* File ):
	m_TexData( NULL ),
	m_DataSize( 0 ),
	m_Width( 0 ),
	m_Height( 0 ),
	m_TileModeS( TileMode_Tile ),
	m_TileModeT( TileMode_Tile ),
	m_EnvMode( EnvMode_Modulate ),
	m_UseMipMap( true ),
	m_Filtering( Filtering_Bilinear ),
	m_CubeMapIndex( -1 ),
	m_OpenGLId( 0 ),
	m_HasChanged( false )
{
	LoadFromFile( File );	
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture constructor
 *
 *  \param  Dup EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
CTexture::CTexture( const CTexture& Dup ):
	m_DataSize( Dup.m_DataSize ),
	m_Width( Dup.m_Width ),
	m_Height( Dup.m_Height ),
	m_TileModeS( Dup.m_TileModeS ),
	m_TileModeT( Dup.m_TileModeT ),
	m_EnvMode( Dup.m_EnvMode ),
	m_UseMipMap( Dup.m_UseMipMap ),
	m_Filtering( Dup.m_Filtering ),
	m_CubeMapIndex( Dup.m_CubeMapIndex ),
	m_OpenGLComponents( Dup.m_OpenGLComponents ),
	m_OpenGLFormat( Dup.m_OpenGLFormat ),
	m_OpenGLType( Dup.m_OpenGLType ),
	m_OpenGLId( 0 ),
	m_HasChanged( false ),
	m_TexData( NULL )
{
	if( Dup.m_TexData )
	{
		m_TexData = new uchar[ m_DataSize ];
		memcpy( m_TexData, Dup.m_TexData, m_DataSize );
	}
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::operator=
 *
 *  \param  Dup EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
const CTexture& CTexture::operator=( const CTexture& Dup )
{
	Reset();
	m_DataSize = Dup.m_DataSize;
	m_Width = Dup.m_Width;
	m_Height = Dup.m_Height;
	m_TileModeS = Dup.m_TileModeS;
	m_TileModeT = Dup.m_TileModeT;
	m_EnvMode = Dup.m_EnvMode;
	m_UseMipMap = Dup.m_UseMipMap;
	m_Filtering = Dup.m_Filtering;
	m_CubeMapIndex = Dup.m_CubeMapIndex;
	m_OpenGLComponents = Dup.m_OpenGLComponents;
	m_OpenGLFormat = Dup.m_OpenGLFormat;
	m_OpenGLType = Dup.m_OpenGLType;
	
	if( Dup.m_TexData )
	{
		m_TexData = new uchar[ m_DataSize ];
		memcpy( m_TexData, Dup.m_TexData, m_DataSize );
	}

	m_HasChanged = true;
	return( *this );
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture destructor
 *
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
CTexture::~CTexture()
{
	Reset();
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::LoadFromFile
 *
 *  \param  File EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
bool CTexture::LoadFromFile( const uchar* File )
{
#ifdef _USE_OPENIL_
	bool ReturnVal = false;
	ILuint Image;
	ilGenImages( 1, &Image );
	ilBindImage( Image );
	ilEnable( IL_ORIGIN_SET );
	ilSetInteger( IL_ORIGIN_MODE, IL_ORIGIN_LOWER_LEFT );

	ILboolean Ok;
	if( File != NULL && ( Ok = ilLoadImage( File ) ) )
	{
		Reset();
		
		if( ilConvertImage( IL_RGBA, IL_UNSIGNED_BYTE ) )
		{
			m_OpenGLComponents = 4;
			m_OpenGLFormat = GL_RGBA;
			m_OpenGLType = GL_UNSIGNED_BYTE;
		}
		else if( ilConvertImage( IL_RGB, IL_UNSIGNED_BYTE ) )
		{
			m_OpenGLComponents = 3;
			m_OpenGLFormat = GL_RGB;
			m_OpenGLType = GL_UNSIGNED_BYTE;
		}
		else
		{
			return false;
		}
		ILinfo Infos;
		iluGetImageInfo( &Infos );
		// Todo : handle when the size is not a power of 2
		m_DataSize = Infos.SizeOfData;
		m_TexData = new uchar[ m_DataSize ];
		memcpy( m_TexData, ilGetData(), m_DataSize );
		m_Width = Infos.Width;
		m_Height = Infos.Height;
		m_HasChanged = true;
		m_ImagePath = File;
		ReturnVal = true;
	}
	ilDeleteImages( 1, &Image );

	return ReturnVal;
#else
	return false;
#endif
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::LoadFromFileInMem
 *
 *  \param  File EXPLANATION
 *  \param  Size EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
bool CTexture::LoadFromFileInMem( const void* File, ulong Size )
{
#ifdef _USE_OPENIL_
	bool ReturnVal = false;
	ILuint Image;
	ilGenImages( 1, &Image );
	ilBindImage( Image );
	ilEnable( IL_ORIGIN_SET );
	ilSetInteger( IL_ORIGIN_MODE, IL_ORIGIN_LOWER_LEFT );
	ILboolean Ok;
	if( Ok = ilLoadL( IL_TYPE_UNKNOWN, (void*)File, Size ) )
	{
		Reset();
		
		if( ilConvertImage( IL_RGBA, IL_UNSIGNED_BYTE ) )
		{
			m_OpenGLComponents = 4;
			m_OpenGLFormat = GL_RGBA;
			m_OpenGLType = GL_UNSIGNED_BYTE;
		}
		else if( ilConvertImage( IL_RGB, IL_UNSIGNED_BYTE ) )
		{
			m_OpenGLComponents = 3;
			m_OpenGLFormat = GL_RGB;
			m_OpenGLType = GL_UNSIGNED_BYTE;
		}
		else
		{
			return false;
		}
		ILinfo Infos;
		iluGetImageInfo( &Infos );
		// Todo : handle when the size is not a power of 2
		m_DataSize = Infos.SizeOfData;
		m_TexData = new uchar[ m_DataSize ];
		memcpy( m_TexData, ilGetData(), m_DataSize );
		m_Width = Infos.Width;
		m_Height = Infos.Height;
		m_HasChanged = true;
		m_ImagePath = "";
		ReturnVal = true;
	}
	ilDeleteImages( 1, &Image );

	return ReturnVal;
#else
	return false;
#endif
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::LoadFromRaw
 *
 *  \param  TexData EXPLANATION
 *  \param  TexDataSize EXPLANATION
 *  \param  Width EXPLANATION
 *  \param  Height EXPLANATION
 *  \param  OpenGLComponents EXPLANATION
 *  \param  OpenGLFormat EXPLANATION
 *  \param  OpenGLType EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
bool CTexture::LoadFromRaw( const uchar* TexData, ulong TexDataSize, ulong Width, ulong Height, GLint  OpenGLComponents, GLenum OpenGLFormat, GLenum OpenGLType )
{
	if( TexData )
	{
		Reset();

		m_DataSize = TexDataSize;
		m_TexData = new uchar[ m_DataSize ];
		memcpy( m_TexData, TexData, m_DataSize );

		m_OpenGLComponents = OpenGLComponents;
		m_OpenGLFormat = OpenGLFormat;
		m_OpenGLType = OpenGLType;
		m_Width = Width;
		m_Height = Height;
		m_HasChanged = true;
		m_ImagePath = "";
		return true;
	}
	return false;
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::Unload
 *
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
bool CTexture::Reset()
{
	if( m_OpenGLId )
	{
		glDeleteTextures( 1, &m_OpenGLId );
		m_OpenGLId = 0;
	}
	DELETEARRAY( m_TexData );
	m_DataSize = 0;
	m_Width = 0;
	m_Height = 0;
	m_ImagePath = "";
	m_HasChanged = true;

	return true;
}



/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::UploadToGL
 *
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
void CTexture::UploadToGL()
{
	if( m_TexData )
	{
		if( m_OpenGLId == 0 )
		{
			glGenTextures( 1, &m_OpenGLId );
			glBindTexture( GL_TEXTURE_2D, m_OpenGLId );
		}

		glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

		// Tiling mode :
		GLint GLTileModeS, GLTileModeT;
		switch( m_TileModeS )
		{
			case TileMode_Clamp: 
				GLTileModeS = GL_CLAMP;
			break;

			case TileMode_ClampToBorder:
				GLTileModeS = GL_CLAMP_TO_BORDER_ARB;
			break;

			case TileMode_Tile :
			default:
				GLTileModeS = GL_REPEAT;
			break;
		}

		switch( m_TileModeT )
		{
			case TileMode_Clamp: 
				GLTileModeT = GL_CLAMP;
			break;

			case TileMode_ClampToBorder:
				GLTileModeT = GL_CLAMP_TO_BORDER_ARB;
			break;

			case TileMode_Tile :
			default:
				GLTileModeT = GL_REPEAT;
			break;
		}	
		
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GLTileModeS );
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GLTileModeT );

		// Mipmap & filtering mode :
		GLint GLMipMap;
		if( m_UseMipMap )
		{
			switch( m_Filtering )
			{
				case Filtering_Anisotropic:
				case Filtering_Trilinear:
					GLMipMap = GL_LINEAR_MIPMAP_LINEAR;
				break;

				case Filtering_Bilinear:
				default:
					GLMipMap = GL_LINEAR_MIPMAP_NEAREST;
				break;
			}
		}
		else
		{
			GLMipMap = GL_NEAREST;
		}

		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GLMipMap );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

		if( m_Filtering == Filtering_Anisotropic )
		{		
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 2.0f );
		}
		else
		{
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 1.0f );
		}
		
		// EnvMode :
		GLint GLEnvMode;
		switch( m_EnvMode )
		{
			case EnvMode_Replace:
				GLEnvMode = GL_REPLACE;
			break;

			case EnvMode_Decal:
				GLEnvMode = GL_DECAL;
			break;

			case EnvMode_Modulate:
			default:
				GLEnvMode = GL_MODULATE;
			break;
		}
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GLEnvMode );


		GLenum GLTarget;
		if( m_CubeMapIndex >= 0 )
		{
			GLTarget = GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT + m_CubeMapIndex;
		}
		else
		{
			GLTarget = GL_TEXTURE_2D;
		}

		if( m_UseMipMap )
		{
			gluBuild2DMipmaps
			(	
				GLTarget	, 		//target
				m_OpenGLComponents,	//nb couleurs
				m_Width,			//largeur
				m_Height,			//hauteur
				m_OpenGLFormat,		//type des couleurs
				m_OpenGLType,		//codage de chaque composante
				m_TexData			//Image
			);
		}
		else
		{
			glTexImage2D
			(
				GLTarget, 			//target
				0,					//mipmap level
				m_OpenGLComponents,	//nb couleurs
				m_Width,			//largeur
				m_Height,			//hauteur
				0,					//border size
				m_OpenGLFormat,		//type des couleurs
				m_OpenGLType,		//codage de chaque composante
				m_TexData			//Image
			);
		}
		m_HasChanged = false;
	}
}


/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::LoadFromScreen
 *
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2004-01-13
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
bool CTexture::LoadFromScreen()
{
	int Params[4];
	glGetIntegerv( GL_VIEWPORT, Params );
	int &x = Params[0];
	int &y = Params[1];
        int &Width = Params[2];
        int &Height = Params[3];

	if( Width != m_Width || Height != m_Height )
	{
		Reset();
		m_DataSize = Width*Height*3;
		m_TexData = new uchar[ m_DataSize ];
		memset( m_TexData, 0, m_DataSize );

		m_UseMipMap = false;
		m_OpenGLComponents = 3;
		m_OpenGLFormat = GL_RGB;
		m_OpenGLType = GL_UNSIGNED_BYTE;
		m_Width = Width;
		m_Height = Height;
		m_HasChanged = false;
		m_ImagePath = "";
	}

	Bind();
	glCopyTexImage2D( GL_TEXTURE_2D, 0, GL_RGB8, x, y, Width, Height, 0 );
	
	return true;
}




/*------------------------------------------------------------------------------*/
/** 
 * Name : CTexture::Bind
 *
 *  \param  -1 EXPLANATION
 *  \return PUT YOUR RETURN VALUE AND ITS EXPLANATION
 *  \author Antoine Bouthors 2002-10-31
 *
 * PUT YOUR COMMENTS HERE
 */ 
/*------------------------------------------------------------------------------*/
void CTexture::Bind()
{
	if( m_OpenGLId == 0 || m_HasChanged )
	{
		UploadToGL();
	}

	if( m_OpenGLId )
	{
		glEnable( GL_TEXTURE_2D );
		if( m_CubeMapIndex >= 0 )
			glBindTexture( GL_TEXTURE_CUBE_MAP_EXT + m_CubeMapIndex, m_OpenGLId );
		else
			glBindTexture( GL_TEXTURE_2D, m_OpenGLId );
	}
	else
	{
		glDisable( GL_TEXTURE_2D );
	}
}


