/*
 *  Camera.cpp
 *  vagues
 *
 *  Created by Nicolas on Tue Dec 02 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */

#include "Camera.h"

/*************************************
 * BASIC CAMERA
 *************************************/
Camera::Camera(GLfloat sx, GLfloat sy,  GLfloat sz, 
               GLfloat vx, GLfloat vy,  GLfloat vz){
    eyePos[0] = sx;
    eyePos[1] = sy;
    eyePos[2] = sz;
    centerPos[0] = vx;
    centerPos[1] = vy;
    centerPos[2] = vz;
    bHasMoved = true;
}

void Camera::moveEyePos(GLfloat x,GLfloat y, GLfloat z){
    eyePos[0] += x;
    eyePos[1] += y;
    eyePos[2] += z;
    this->bHasMoved = true;
}
void Camera::setEyePos(GLfloat x,GLfloat y, GLfloat z){
    eyePos[0] = x;
    eyePos[1] = y;
    eyePos[2] = z;
    this->bHasMoved = true;
}

void Camera::moveCenterPos(GLfloat cx,GLfloat cy, GLfloat cz){
    centerPos[0] += cx;
    centerPos[1] += cy;
    centerPos[2] += cz;
    this->bHasMoved = true;
}
void Camera::setCenterPos(GLfloat cx,GLfloat cy, GLfloat cz){
    centerPos[0] = cx;
    centerPos[1] = cy;
    centerPos[2] = cz;
    this->bHasMoved = true;
}

#include <iostream>
void Camera::drawMe(){
    gluLookAt(eyePos[0],eyePos[1],eyePos[2],centerPos[0],centerPos[1],centerPos[2],0.0,1.0,0.0);
    this->bHasMoved = false;
}


/*************************************
* MOVEABLE CAMERA
*************************************/
MoveableCamera::MoveableCamera(GLfloat sx, GLfloat sy,  GLfloat sz, 
                               GLfloat vx, GLfloat vy,  GLfloat vz):
Camera(sx,sy,sz,vx,vy,vz),
bDisplayPov(false)
{}

//MOVE THE EYE POSITION
void MoveableCamera::setEyePosDirection(GLfloat thetaEye,GLfloat phiEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                         (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                         (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    eyePos[0] = centerPos[0] + dist*cos(thetaEye)*cos(phiEye);
    eyePos[2] = centerPos[2] + dist*sin(thetaEye)*cos(phiEye);
    eyePos[1] = centerPos[1] + dist*sin(phiEye);    
    this->bHasMoved = true;
}
void MoveableCamera::moveEyePosDirection(GLfloat thetaEye,GLfloat phiEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                         (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                         (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    GLfloat phi = asin((eyePos[1]-centerPos[1])/dist);
    GLfloat theta = asin((eyePos[2]-centerPos[2])/cos(phi)/dist);
    if(eyePos[0]<centerPos[0]){
        theta = -theta + M_PI;
    }
    theta += thetaEye;
    phi += phiEye;
    eyePos[0] = centerPos[0] + dist*cos(theta)*cos(phi);
    eyePos[2] = centerPos[2] + dist*sin(theta)*cos(phi);
    eyePos[1] = centerPos[1] + dist*sin(phi);    
    this->bHasMoved = true;
}
void MoveableCamera::setEyePosDistance(GLfloat distEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    GLfloat phi = asin((eyePos[1]-centerPos[1])/dist);
    GLfloat theta = asin((eyePos[2]-centerPos[2])/cos(phi)/dist);
    if(eyePos[0]<centerPos[0]){
        theta = -theta + M_PI;
    }
    dist = distEye;
    eyePos[0] = centerPos[0] + dist*cos(theta)*cos(phi);
    eyePos[2] = centerPos[2] + dist*sin(theta)*cos(phi);
    eyePos[1] = centerPos[1] + dist*sin(phi);    
    this->bHasMoved = true;
}
void MoveableCamera::moveEyePosDistance(GLfloat distEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    GLfloat phi = asin((eyePos[1]-centerPos[1])/dist);
    GLfloat theta = asin((eyePos[2]-centerPos[2])/cos(phi)/dist);
    if(eyePos[0]<centerPos[0]){
        theta = -theta + M_PI;
    }
    dist += distEye;
    eyePos[0] = centerPos[0] + dist*cos(theta)*cos(phi);
    eyePos[2] = centerPos[2] + dist*sin(theta)*cos(phi);
    eyePos[1] = centerPos[1] + dist*sin(phi);
    this->bHasMoved = true;
}


//MOVE THE CENTER OF VIEW
void MoveableCamera::setCenterPosDirection(GLfloat thetaEye,GLfloat phiEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    centerPos[0] = eyePos[0] + dist*cos(thetaEye)*cos(phiEye);
    centerPos[2] = eyePos[2] + dist*sin(thetaEye)*cos(phiEye);
    centerPos[1] = eyePos[1] + dist*sin(phiEye);    
    this->bHasMoved = true;
}
void MoveableCamera::moveCenterPosDirection(GLfloat thetaEye,GLfloat phiEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    GLfloat phi = asin((centerPos[1]-eyePos[1])/dist);
    GLfloat theta = asin((centerPos[2]-eyePos[2])/cos(phi)/dist);
    if(centerPos[0]<eyePos[0]){
        theta = -theta + M_PI;
    }

    theta += thetaEye;
    phi += phiEye;
    centerPos[0] = eyePos[0] + dist*cos(theta)*cos(phi);
    centerPos[2] = eyePos[2] + dist*sin(theta)*cos(phi);
    centerPos[1] = eyePos[1] + dist*sin(phi);
    this->bHasMoved = true;
}
//Move and set the distance of the 
/*void MoveableCamera::setCenterPosDistance(GLfloat distEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    GLfloat phi = asin((centerPos[2]-eyePos[2])/dist);
    GLfloat theta = asin((centerPos[1]-eyePos[1])/cos(phi)/dist);
    dist = distEye;
    if(centerPos[0]<eyePos[0]){
        theta = -theta + M_PI;
    }    
    centerPos[0] = eyePos[0] + dist*cos(theta)*cos(phi);
    centerPos[1] = eyePos[1] + dist*sin(theta)*cos(phi);
    centerPos[2] = eyePos[2] + dist*sin(phi);    
}
void MoveableCamera::moveCenterPosDistance(GLfloat distEye){
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]));
    GLfloat phi = asin((centerPos[2]-eyePos[2])/dist);
    GLfloat theta = asin((centerPos[1]-eyePos[1])/cos(phi)/dist);
    dist += distEye;
    if(centerPos[0]<eyePos[0]){
        theta = -theta + M_PI;
    }
    centerPos[0] = eyePos[0] + dist*cos(theta)*cos(phi);
    centerPos[1] = eyePos[1] + dist*sin(theta)*cos(phi);
    centerPos[2] = eyePos[2] + dist*sin(phi);
}*/

void MoveableCamera::moveWalk(short walkMode, GLfloat _step){
    GLfloat vec[2];
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                        (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]))/_step;
    for (int i=0;i<3;i++)
        vec[i] = (centerPos[i]-eyePos[i])/dist;
    switch(walkMode){
        case WALK_FORWARD :             
            this->moveEyePos(vec[0],0,vec[2]);
            this->moveCenterPos(vec[0],0,vec[2]);
            break;
        case WALK_BACKWARD : 
            this->moveEyePos(-vec[0],0,-vec[2]);
            this->moveCenterPos(-vec[0],0,-vec[2]);
            break;
        case WALK_LEFT :
            this->moveEyePos(vec[2],0,-vec[0]);
            this->moveCenterPos(vec[2],0,-vec[0]);
            break;                    
        case WALK_RIGHT :
            this->moveEyePos(-vec[2],0,vec[0]);
            this->moveCenterPos(-vec[2],0,vec[0]);
            break;
    }
}
void MoveableCamera::moveFly(short flyMode, GLfloat _step){
    GLfloat vec[3];
    GLfloat dist = sqrt((eyePos[0]-centerPos[0])*(eyePos[0]-centerPos[0])+
                (eyePos[1]-centerPos[1])*(eyePos[1]-centerPos[1])+
                (eyePos[2]-centerPos[2])*(eyePos[2]-centerPos[2]))/_step;
    for (int i=0;i<3;i++)
        vec[i] = (centerPos[i]-eyePos[i])/dist;
    switch(flyMode){
        case FLY_FORWARD :             
            this->moveEyePos(vec[0],vec[1],vec[2]);
            this->moveCenterPos(vec[0],vec[1],vec[2]);
            break;
        case FLY_BACKWARD : 
            this->moveEyePos(-vec[0],-vec[1],-vec[2]);
            this->moveCenterPos(-vec[0],-vec[1],-vec[2]);
            break;
        case FLY_LEFT :
            this->moveEyePos(vec[2],0,-vec[0]);
            this->moveCenterPos(vec[2],0,-vec[0]);
            break;                    
        case FLY_RIGHT :
            this->moveEyePos(-vec[2],0,vec[0]);
            this->moveCenterPos(-vec[2],0,vec[0]);
            break;
    }
}
/*
 * Draw Method
 */
void MoveableCamera::drawMe(){
    this->Camera::drawMe();
    if(this->bDisplayPov){
        glPushMatrix();
        glPushAttrib(GL_LIGHTING_BIT);
        glDisable(GL_LIGHTING);
        glColor3fv(fullred);
        glTranslatef(this->centerPos[0],this->centerPos[1],this->centerPos[2]);
        glutWireSphere(.1,6,6);
        glPopAttrib();
        glPopMatrix();
    }
}


/*************************************
* CONTROLLED CAMERA
*************************************/
#define KBCONTROL_MENU_ENTRY 0
#define CYCLECAM_MENU_ENTRY 1
#define DISPCENTER_MENUENTRY 2
ControlledCamera::ControlledCamera(short camMode , GLfloat sx ,GLfloat sy ,GLfloat sz ,
                                   GLfloat vx ,GLfloat vy , GLfloat vz ):
MoveableCamera(sx,sy,sz,vx,vy,vz),
cameraMode(camMode)
{

    this->camMenu = new Menu("Camera");
    StaticMenu::currentMenu->addSubMenu(camMenu);
    camMenu->addSubMenu(new Menu("Controler au clavier",this, KBCONTROL_MENU_ENTRY));
    camMenu->addSubMenu(new Menu("Changer de mode de camera",this, CYCLECAM_MENU_ENTRY));
    camMenu->addSubMenu(new Menu("Afficher le point de visee",this, DISPCENTER_MENUENTRY));
}

void ControlledCamera::activeMotionFunc(int x, int y){
    switch(this->cameraMode){
        case CAMERA_FLY:
            this->moveCenterPosDirection((x-Mouse::getOldMouseX())/100.0,(y-Mouse::getOldMouseY())/-100.0);
            break;
        case CAMERA_WALK:
            this->moveCenterPosDirection((x-Mouse::getOldMouseX())/100.0,(y-Mouse::getOldMouseY())/-100.0);
            break;
        case CAMERA_EXAMINE:
            this->moveEyePosDirection((x-Mouse::getOldMouseX())/-100.0,(y-Mouse::getOldMouseY())/-100.0);
            break;
    }
    glutPostRedisplay();
}

void ControlledCamera::keyboardSpecialFunc(int c, int x, int y){
    switch(this->cameraMode){
        case CAMERA_FLY:
            switch(c){
                case GLUT_KEY_UP : this->moveFly(FLY_FORWARD,1); break;
                case GLUT_KEY_DOWN : this->moveFly(FLY_BACKWARD,1); break;
                case GLUT_KEY_LEFT : this->moveFly(FLY_LEFT,1); break;                    
                case GLUT_KEY_RIGHT : this->moveFly(FLY_RIGHT,1); break;
            }
            break;
        case CAMERA_WALK:
            switch(c){
                case GLUT_KEY_UP : this->moveWalk(WALK_FORWARD,1); break;
                case GLUT_KEY_DOWN : this->moveWalk(WALK_BACKWARD,1); break;
                case GLUT_KEY_LEFT : this->moveWalk(WALK_LEFT,1); break;                    
                case GLUT_KEY_RIGHT : this->moveWalk(WALK_RIGHT,1); break;
            }
            break;
        case CAMERA_EXAMINE:
            switch(c)
            {
                case GLUT_KEY_UP : this->moveEyePosDistance(-1);break;
                case GLUT_KEY_DOWN : this->moveEyePosDistance(1);break;
                case GLUT_KEY_LEFT : this->moveEyePosDirection(0.1,0);break;
                case GLUT_KEY_RIGHT :this->moveEyePosDirection(-0.1,0);break;
            }
            break;
    }
    glutPostRedisplay();
}

void ControlledCamera::menuItemClicked(int menuEntry){
    switch(menuEntry){
        case KBCONTROL_MENU_ENTRY:
            hud::currentHud->addMessage(new TemporizedMessage("Utilisez les touches fleches pour deplacer la camera"));
            Keyboard::selectKeyboardMode(this);
            break;
        case CYCLECAM_MENU_ENTRY:
            switch(this->cameraMode){
                case CAMERA_FLY: this->cameraMode = CAMERA_WALK;
                    hud::currentHud->addMessage(new TemporizedMessage("Deplacement en mode WALK"));
                    break;
                case CAMERA_WALK: this->cameraMode = CAMERA_EXAMINE;
                    hud::currentHud->addMessage(new TemporizedMessage("Deplacement en mode EXAMINE"));
                    this->bDisplayPov= true;
                    hud::currentHud->addMessage(new TemporizedMessage("Affichage du point de visee active"));
                    break;
                case CAMERA_EXAMINE: this->cameraMode = CAMERA_FLY;
                    hud::currentHud->addMessage(new TemporizedMessage("Deplacement en mode FLY"));
                    break;
            }
            Keyboard::selectKeyboardMode(this);
            break;
        case DISPCENTER_MENUENTRY:
            if(this->bDisplayPov){
                this->bDisplayPov= false;
                hud::currentHud->addMessage(new TemporizedMessage("Affichage du point de visee desactive"));
            }else{
                this->bDisplayPov= true;
                hud::currentHud->addMessage(new TemporizedMessage("Affichage du point de visee active"));
            }
            break;
        default:break;
    }
}

