/*
 *  Camera.h
 *  vagues
 *
 *  Created by Nicolas on Tue Dec 02 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 *
 * A simple abstraction of OpenGL Cameras.
 * Plus, some stuff to move it.
 *
 *
 * The camera class has a simple function with Mouse class to move pov around 0 when active motion, 
 * but you can change it by inheritance. 
*/

#ifndef _Camera_h_
#define _Camera_h_
#ifdef __APPLE__
#include <glut.h>
#else
#include <GL/glut.h>
#endif
#include <math.h>
#include "constantes.h"
#include "graphicElement.h"
#include "Mouse.h"
#include "Keyboard.h"
#include "Menu.h"
#include "hud.h"

class Camera:public graphicElement{
public:
    Camera(GLfloat sx = 0,GLfloat sy = 0, GLfloat sz = 0,
           GLfloat vx = 0,GLfloat vy = 0, GLfloat vz = 0);

    void setEyePos(GLfloat x,GLfloat y, GLfloat z);
    void moveEyePos(GLfloat x,GLfloat y, GLfloat z);
    void setCenterPos(GLfloat cx,GLfloat cy, GLfloat cz);
    void moveCenterPos(GLfloat cx,GLfloat cy, GLfloat cz);

    GLfloat getX(){return eyePos[0];};
    GLfloat getY(){return eyePos[1];};
    GLfloat getZ(){return eyePos[2];};
    
    bool hasMoved(){return bHasMoved;};

protected:
        virtual void drawMe();
    GLfloat eyePos[3];
    GLfloat centerPos[3];
    
    bool bHasMoved;

};



/*
 * A Camera that allows FLY, WALK, EXAMINE
 */
#define WALK_FORWARD 0
#define WALK_BACKWARD 1
#define WALK_LEFT 2
#define WALK_RIGHT 3
#define FLY_FORWARD 4
#define FLY_BACKWARD 5
#define FLY_LEFT 6
#define FLY_RIGHT 7
class MoveableCamera:public Camera{
public:
    MoveableCamera(GLfloat sx = 0,GLfloat sy = 0, GLfloat sz = 1,
                   GLfloat vx = 0,GLfloat vy = 0, GLfloat vz = 0);
    void setEyePosDirection(GLfloat th,GLfloat ph);
    void moveEyePosDirection(GLfloat th,GLfloat ph);
    void setEyePosDistance(GLfloat dist);
    void moveEyePosDistance(GLfloat dist);
    
    void setCenterPosDirection(GLfloat th,GLfloat ph);
    void moveCenterPosDirection(GLfloat th,GLfloat ph);
    /* 
        * Should be totally useless.
     void setCenterPosDistance(GLfloat dist);
     void moveCenterPosDistance(GLfloat dist);
     */    
    
    void moveWalk(short walkMode, GLfloat _step);
    void moveFly(short flyMode, GLfloat _step);
    
protected:
    bool bDisplayPov;
        virtual void drawMe();

};


/*
 * A Camera that supports Mouse and Keyboard and contextual-menu control
 */
#define CAMERA_FLY 0
#define CAMERA_WALK 1
#define CAMERA_EXAMINE 2
class ControlledCamera:public MoveableCamera, public MouseMode, public KeyboardMode, public MenuItemHandler{
    short cameraMode;
    Menu * camMenu;
public:    
        ControlledCamera(short camMode = CAMERA_FLY, GLfloat sx = 0,GLfloat sy = 0, GLfloat sz = 1,
                         GLfloat vx = 0,GLfloat vy = 0, GLfloat vz = 0);
    virtual void activeMotionFunc(int x, int y);
    virtual void keyboardSpecialFunc(int c, int x, int y);
    virtual void menuItemClicked(int MenuEntry);
    
};

#endif


