/*
 *  Keyboard.h
 *  Waves
 *
 *  Created by Nicolas on Sat Nov 29 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 * ajouter glutGetModifiers?
 *
 *
 * Keyboard is a combination of 2 classes:
 * A static class Keyboard which provides high-level integration.
 * A abstract class KeyboardMode which can be inherited by user's objects.
 *
 * This mechanism is similar to the Mouse mechanism.
 */


#ifndef _Keyboard_h_
#define _Keyboard_h_


#ifdef __APPLE__
#include <glut.h>
#else
#include <GL/glut.h>
#endif

/*
 * Interface to allow user's objects to respond to keyboard events.
 * the descendent class does not have to reimplement all the functions.
 * The static Mouse function will call the correct function when event rise.
 * Mopreover, this abstract class provides function to set autorepeat on the keyboard
 * on or off.
 * 
 */
class KeyboardMode{
    friend class  Keyboard;
    bool bKeyRepeat;
    bool bKeyDefaultCompatible;
public:
        KeyboardMode(){bKeyRepeat = true;bKeyDefaultCompatible = true;};
protected:
        void setKeyRepeat(bool bKR){ bKeyRepeat = bKR;};
    void setDefaultModeCompatibility(bool bKC){ bKeyDefaultCompatible = bKC;};
    virtual void keyboardFunc(unsigned char c, int x, int y){};
    virtual void keyboardUpFunc(unsigned char c, int x, int y){};
    virtual void keyboardSpecialFunc(int c, int x, int y){};
    virtual void keyboardSpecialUpFunc(int c, int x, int y){};
};


/*
 * The static Keyboard class provides static keyboardfunctions for GLUT
 * and relays the event to the currentKeyboardMode chose by the user and
 * selected by selectKeyboardMode. There is an additionnal mechanism for
 * a default KeyboardMode to call when you want to exit a specific KeyboardMode.
 *
 * This class may include a function to map glutGetModifers in the future in order to 
 * get the command key on the mac keyboard.
 */

class Keyboard{
    static KeyboardMode* currentKbMode;
    static KeyboardMode* defaultKbMode;
public:
    /*
     * These functions are to select the current Object that will respond to Mouse events.
     */
    static void selectDefaultKeyboardMode();
    static void setDefaultKeyboardMode(KeyboardMode* kbm);
    static void selectKeyboardMode(KeyboardMode* kbm);

private:
    static void linkStaticFunctions();
    static bool bStaticFunctionsLinked;
    static void staticKeyboardFunc(unsigned char c, int x, int y);
    static void staticKeyboardUpFunc(unsigned char c, int x, int y);
    static void staticKeyboardSpecialFunc(int c, int x, int y);
    static void staticKeyboardSpecialUpFunc(int c, int x, int y);
};

#endif

