/*
 *  Menu.h
 *  vagues
 *
 *  Created by Nicolas on Thu Jan 15 2004.
 *  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __Menu_h__
#define __Menu_h__

#ifdef WIN32
# pragma warning( disable: 4786 )
#endif
#ifdef __APPLE__
#include <glut.h>
#else
#include <GL/glut.h>
#endif
#include <map>
#include <vector>
#include <string>
#include <algorithm>
using std::map;
using std::vector;
using std::string;
/*
 *Interface to be implemented by user's classes that will handle menu actions
 */
class MenuItemHandler{
public:
    virtual void menuItemClicked(int MenuEntry)=0;
};


/*
 * A menu element: can be a submenu header or a single leaf.
 * the label is the label of the menu,
 * and the handler is the object that contains the menuItemClicked method that will handle that menuitem's actions.
 * NOTA: with glut, menu that contain sub-menuitems are never clicked. (ie, you cant specify an action for it)
 *
 * Once your menuItem is created, you need to add to its parent menuItem by using addSubMenu. BUT you must be sure 
 * that there is a root menuItem (of class menu) at the top of it. If you do well, 
 * this method returns you an integer, which is the action code that will be sent to your MenuItemHandler's
 * menuItemClicked method, so you can know what item is clicked without havong to create a handler for 
 * each item.
 *
 * When the object that corresponds to the menuItem is deleted, be sure to remove it 
 * from its parent menu by calling removeSubMenu.
 *
 * Moreover, users can specify the button that will be used for the menu.
 * it can be: 
 * GLUT_LEFT_BUTTON
 * GLUT_MIDDLE_BUTTON
 * GLUT_RIGHT_BUTTON
 * But pay attention to the fact that only the mousebutton specified for the root menu is meaningful.
 */
class Menu{
    //friend class Menu;
    friend class StaticMenu;
public:
        Menu(const string& _label, MenuItemHandler* _handler = NULL,
                 int _handlerMenuEntryID = -1, int _mouseButton = GLUT_RIGHT_BUTTON);
     void addSubMenu(Menu * newMenu);
     void removeSubMenu(Menu * oldMenu);
protected:
        virtual ~Menu();
    string myLabel;
    MenuItemHandler * myHandler;
    GLint myGlutID;
    int handlerMenuEntryID;

    int mouseButton;
    
    vector<Menu*> mySubMenus;
    virtual void buildGLUTmenu();
    void shoot();
};

/*
 * The static part that glue with GLUT :-)
 * For the user, it contains a default Menu currentMenu initialized at startup.
 * the default currentMenu is created with action button = GLUT_RIGHT_BUTTON
 *
 * The main job of this class is to keep the map menuGlutID->menu. When a menu is clicked,
 * glut calls StaticMenu's processMenuEvents which in return calls shoot() on the menu
 * found in his map corresponding to the menuGlutID
 */
class StaticMenu{
    friend class Menu;
    static void createGLUTmenu();
    static void processMenuEvents(int menuEntry);
public:
     static Menu* currentMenu;
private:
        static map<int,Menu*> myMenus;
    static int handlerCounter;
    static int addToMap(Menu* newMenu);
};
#endif

