/*
 *  Mouse.h
 *  Waves
 *
 *  Created by Nicolas on Sun Nov 30 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 * Mouse is a combination of 2 classes:
 * A static class Mouse which provides high-level integration.
 * A abstract class MouseMode which can be inherited by user's objects.
 *
 * This mechanism is similar to the Keyboard mechanism.
 */

#ifndef _Mouse_h_
#define _Mouse_h_

#ifdef __APPLE__
#include <glut.h>
#else
#include <GL/glut.h>
#endif

/*
 * Interface to allow user's objects to respond to mouse events.
 * the descendent class does not have to reimplement all the functions.
 * The static Mouse function will call the correct function when event rise.
 */
class MouseMode{
    friend class Mouse;
    bool bMouseDefaultCompatible;
public:
    MouseMode(){bMouseDefaultCompatible = true;};
    void setDefaultModeCompatibility(bool mBC){bMouseDefaultCompatible = mBC;};
    virtual void mouseFunc(int button, int state, int x, int y){};
    virtual void activeMotionFunc(int x, int y){};
    virtual void passiveMotionFunc(int x, int y){};
    virtual void mouseEntryFunc(int state){};
};

/*
 * The static Mouse class provides static mousefunctions for GLUT
 * and relays the event to the currentMouseMode chose by the user and
 * selected by selectMouseMode. There is an additionnal mechanism for
 * a default MouseMode to call when you want to exit a specific MouseMode.
 */
class Mouse{
    static MouseMode* currentMMode;
    static MouseMode* defaultMMode;
    static GLint oldMouseX, oldMouseY;
 
public:
    /*
     * These functions are to select the current Object that will respond to Mouse events.
     */
    static void selectMouseMode(MouseMode* mm);
    static void selectDefaultMouseMode();
    static void setDefaultMouseMode(MouseMode* mm);

    /*
     *Additionnally, Mouse provides functions to get the last-click coordinates.
     * I.E. the last place where button was down (clicked or dragged)
     */
    static GLint getOldMouseX();
    static GLint getOldMouseY();
    
private:
    static void linkStaticFunctions();
    static bool bStaticFunctionsLinked;
    static void staticMouseFunc(int button, int state, int x, int y);
    static void staticActiveMotionFunc(int x, int y);
    static void staticPassiveMotionFunc(int x, int y);
    static void staticMouseEntryFunc(int state);
};


#endif


