/*
 *  graphicsElements.h
 *  opengl2
 *
 *  Created by Nicolas on Thu Nov 20 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 *
 * This is a general container for drawable objects that may contain others.
 * as a logical meaning (it does not mean that "sons" objects are graphical
 * parts of the parent object)
 *
 * This is an abstract class (java-speaking) and will not be instanciated
 */

#ifndef _graphicElement_h_
#define _graphicElement_h_

#include <vector>
#include <algorithm>
using std::vector;

class graphicElement{
    /*
     * sons are kept in a vector.
     */
    vector<graphicElement*> sons;
public:
    virtual ~graphicElement(){};
    /*
     * Simple assessors to the sons vector
     */
    void removeSon(graphicElement* e);
    void addSon(graphicElement* e);
    /*
     * draw method.
     * calls itself on every son of this graphicElement
     * and calls the drawMe protected method of the graphicElement on which it is called
     * descendant class should not define their own draw routine 
     * (normally, since draw is not virtual it should result in a name clash but it doesnt always AFAIK)
     */
    void draw();
protected:
        /*
         * drawMe method should be redifined by inherited classes to perform the main "draw" stuff.
         */
        virtual void drawMe()=0;
};

#endif

