/*
 *  hud.h
 *  opengl2
 *
 *  Created by Nicolas on Mon Nov 17 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 * The hud provides simple ways to output text onscreen.
 * Messages: can specify a location (x,y) and a duration 
 * or the message can ben displayed only for 1 frame, and then removed.
 */

class hud;
#ifndef DEFINE_HUD
#define DEFINE_HUD

#ifndef __APPLE__
#include <GL/glut.h>
#else
#include <glut.h>
#endif
#include "constantes.h"
#include "graphicElement.h"
#include "../SimuParams.h"

#include <string>
using std::string;
//forward declaration
class Message;


/*
 * The HUD class provides a simple way to output messages on-screen.
 * The user adds Message, which are created from screen
 * The HUD then displays its messages on one frame and deletes the messages that are no-longer needed.
 */
class hud: public virtual graphicElement{
    vector<Message*> messages;
public:
    static hud* currentHud;
    hud() {};
    virtual void addMessage(Message * m);
    virtual void clearMessages();
    void deleteMessage(Message* m);
protected:
        virtual void drawMe();
};


/*
 * This class provides a way to display single-frame messages that will be drawn at location (x,y)
 */
class Message{
    friend class hud;
public:
    Message(const string& m,int x, int y);
protected:
    Message(){};
    virtual ~Message(){};
    virtual bool isDead() {return true;};
    virtual void drawMessage();
    string msg;
    int x;
    int y;
};

/*
 * This class allows the user to create messages that are displayed for *duration* milliseconds in a list.
 * The duration is the same for all messages in the list.
 * it can be changed.
 * Currently this list is in the lower-left corner, it should be changed.
 */
class TemporizedMessage:public Message{
public:
    TemporizedMessage(const string& m);
protected:
    virtual void drawMessage();
    virtual ~TemporizedMessage(){firstMessageNumber++;};
    virtual bool isDead();
    GLint endTime;
    int messageNumber;
    static int firstMessageNumber;
    static int lastMessageNumber;
    static GLint duration;
};

/*
 *
 */
class PermanentMessage:public Message{
public:
    PermanentMessage(const string& m,int x, int y);
    void die(){dead = true;};
    void setMsg(const string& m);
protected:
    bool dead;
    virtual ~PermanentMessage(){};
    virtual bool isDead(){return dead;};
};
#endif



